"""
The mappings are based of https://www.thecrag.com/en/article/grades,
which contains all grades in a form of tables.
"""

from dataclasses import dataclass


@dataclass(frozen=True)
class GradeInfo:
    start: int
    height: int

    @property
    def end(self) -> int:
        return self.start + self.height


VALID_NAMES = (
    ("Band Sport",),
    ("Ewbanks", "AU", "Australia", "New Zealand"),
    ("YDS", "Yosemitee", "American", "US"),
    ("NCCS Scale", "Commitment Grade"),
    ("French", "FR"),
    ("British Tech.", "UK", "British"),
    ("UIAA",),
    ("South African", "SA"),
    ("Old South African", "OSA"),
    ("Saxon", "SX"),
    ("Finnish", "FIN"),
    ("Norwegian", "NWG"),
    ("Polish", "PL", "Kurtyki"),
    ("Brazil Technical", "BRZ", "Brazilian"),
    ("Swedish", "SWE"),
    ("Russian", "RU"),
    ("Band Boulder",),
    ("V-Scale", "V"),
    ("B-Scale", "B"),
    ("S-Scale", "S"),
    ("P-Scale", "P"),
    ("Joshua Tree Scale", "Joshua"),
    ("Fontainebleau", "Font"),
    ("Annot B-Scale",),
    ("Font Traverse",),
    ("Band Other",),
    ("Aid",),
    ("Alpine Ice",),
    ("Water Ice",),
    ("Mixed Rock/Ice",),
    ("Ferrata Schall",),
    ("Ferrata Num",),
    ("Ferrata French",),
    ("Scottish Winter Technical",),
)

BOULDER_SYSTEMS = (
    "Band Boulder",
    "V-Scale",
    "B-Scale",
    "S-Scale",
    "P-Scale",
    "Joshua Tree Scale",
    "Fontainebleau",
    "Annot B-Scale",
    "Font Traverse",
)

OTHER_SYSTEMS = (
    "Band Other",
    "Aid",
    "Alpine Ice",
    "Water Ice",
    "Mixed Rock/Ice",
    "Ferrata Schall",
    "Ferrata Num",
    "Ferrata French",
    "Scottish Winter Technical",
)

NAMING_MAP = {}
for system in VALID_NAMES:
    for name in system:
        NAMING_MAP[name.lower()] = system[0]

SPORT_MAPPING = {
    "Band Sport": {
        "Beginner": GradeInfo(0, 300),
        "Intermediate": GradeInfo(300, 300),
        "Experienced": GradeInfo(600, 300),
        "Expert": GradeInfo(900, 300),
        "Elite": GradeInfo(1200, 450),
    },
    "Ewbanks": {
        "1": GradeInfo(0, 24),
        "2": GradeInfo(24, 27),
        "3": GradeInfo(51, 24),
        "4": GradeInfo(75, 24),
        "5": GradeInfo(99, 27),
        "6": GradeInfo(126, 24),
        "7": GradeInfo(150, 24),
        "8": GradeInfo(174, 27),
        "9": GradeInfo(201, 24),
        "10": GradeInfo(225, 24),
        "11": GradeInfo(249, 27),
        "12": GradeInfo(276, 24),
        "13": GradeInfo(300, 51),
        "14": GradeInfo(351, 48),
        "15": GradeInfo(399, 51),
        "16": GradeInfo(450, 51),
        "17": GradeInfo(501, 48),
        "18": GradeInfo(549, 51),
        "19": GradeInfo(600, 51),
        "20": GradeInfo(651, 48),
        "21": GradeInfo(699, 51),
        "22": GradeInfo(750, 51),
        "23": GradeInfo(801, 48),
        "24": GradeInfo(849, 51),
        "25": GradeInfo(900, 39),
        "26": GradeInfo(939, 36),
        "27": GradeInfo(975, 39),
        "28": GradeInfo(1014, 36),
        "29": GradeInfo(1050, 39),
        "30": GradeInfo(1089, 36),
        "31": GradeInfo(1125, 39),
        "32": GradeInfo(1164, 36),
        "33": GradeInfo(1200, 42),
        "34": GradeInfo(1242, 45),
        "35": GradeInfo(1287, 42),
        "36": GradeInfo(1329, 42),
        "37": GradeInfo(1371, 42),
        "38": GradeInfo(1413, 45),
        "39": GradeInfo(1458, 42),
    },
    "YDS": {
        "5.0": GradeInfo(66, 39),
        "5.1": GradeInfo(105, 39),
        "5.2": GradeInfo(144, 39),
        "5.3": GradeInfo(183, 39),
        "5.4": GradeInfo(222, 39),
        "5.5": GradeInfo(261, 39),
        "5.6": GradeInfo(300, 60),
        "5.7": GradeInfo(360, 60),
        "5.8": GradeInfo(420, 60),
        "5.9": GradeInfo(480, 60),
        "5.10a": GradeInfo(540, 60),
        "5.10b": GradeInfo(600, 39),
        "5.10c": GradeInfo(639, 36),
        "5.10d": GradeInfo(675, 39),
        "5.11a": GradeInfo(714, 36),
        "5.11b": GradeInfo(750, 39),
        "5.11c": GradeInfo(789, 36),
        "5.11d": GradeInfo(825, 39),
        "5.12a": GradeInfo(864, 36),
        "5.12b": GradeInfo(900, 39),
        "5.12c": GradeInfo(939, 36),
        "5.12d": GradeInfo(975, 39),
        "5.13a": GradeInfo(1014, 36),
        "5.13b": GradeInfo(1050, 39),
        "5.13c": GradeInfo(1089, 36),
        "5.13d": GradeInfo(1125, 39),
        "5.14a": GradeInfo(1164, 36),
        "5.14b": GradeInfo(1200, 42),
        "5.14c": GradeInfo(1242, 45),
        "5.14d": GradeInfo(1287, 42),
        "5.15a": GradeInfo(1329, 42),
        "5.15b": GradeInfo(1371, 42),
        "5.15c": GradeInfo(1413, 45),
        "5.15d": GradeInfo(1458, 42),
    },
    "NCCS Scale": {
        "F4": GradeInfo(0, 150),
        "F5": GradeInfo(150, 150),
        "F6": GradeInfo(300, 75),
        "F7": GradeInfo(375, 75),
        "F8": GradeInfo(450, 75),
        "F9": GradeInfo(525, 75),
        "F10": GradeInfo(600, 60),
        "F11": GradeInfo(660, 60),
        "F12": GradeInfo(720, 60),
        "F13": GradeInfo(780, 60),
        "F14": GradeInfo(840, 60),
        "F15": GradeInfo(900, 75),
        "F16": GradeInfo(975, 75),
    },
    "French": {
        "1a": GradeInfo(0, 18),
        "1a+": GradeInfo(18, 15),
        "1b": GradeInfo(33, 18),
        "1b+": GradeInfo(51, 15),
        "1c": GradeInfo(66, 18),
        "1c+": GradeInfo(84, 15),
        "2a": GradeInfo(99, 18),
        "2a+": GradeInfo(117, 15),
        "2b": GradeInfo(132, 18),
        "2b+": GradeInfo(150, 18),
        "2c": GradeInfo(168, 15),
        "2c+": GradeInfo(183, 18),
        "3a": GradeInfo(201, 15),
        "3a+": GradeInfo(216, 18),
        "3b": GradeInfo(234, 15),
        "3b+": GradeInfo(249, 18),
        "3c": GradeInfo(267, 15),
        "3c+": GradeInfo(282, 18),
        "4a": GradeInfo(300, 24),
        "4a+": GradeInfo(324, 21),
        "4b": GradeInfo(345, 24),
        "4b+": GradeInfo(369, 24),
        "4c": GradeInfo(393, 21),
        "4c+": GradeInfo(414, 24),
        "5a": GradeInfo(438, 24),
        "5a+": GradeInfo(462, 24),
        "5b": GradeInfo(486, 21),
        "5b+": GradeInfo(507, 24),
        "5c": GradeInfo(531, 24),
        "5c+": GradeInfo(555, 21),
        "6a": GradeInfo(576, 24),
        "6a+": GradeInfo(600, 42),
        "6b": GradeInfo(642, 45),
        "6b+": GradeInfo(687, 42),
        "6c": GradeInfo(729, 42),
        "6c+": GradeInfo(771, 42),
        "7a": GradeInfo(813, 45),
        "7a+": GradeInfo(858, 42),
        "7b": GradeInfo(900, 39),
        "7b+": GradeInfo(939, 36),
        "7c": GradeInfo(975, 39),
        "7c+": GradeInfo(1014, 36),
        "8a": GradeInfo(1050, 39),
        "8a+": GradeInfo(1089, 36),
        "8b": GradeInfo(1125, 39),
        "8b+": GradeInfo(1164, 36),
        "8c": GradeInfo(1200, 42),
        "8c+": GradeInfo(1242, 45),
        "9a": GradeInfo(1287, 42),
        "9a+": GradeInfo(1329, 42),
        "9b": GradeInfo(1371, 42),
        "9b+": GradeInfo(1413, 45),
        "9c": GradeInfo(1458, 42),
    },
    "British Tech.": {
        "1a": GradeInfo(0, 30),
        "1b": GradeInfo(30, 30),
        "1c": GradeInfo(60, 30),
        "2a": GradeInfo(90, 30),
        "2b": GradeInfo(120, 30),
        "2c": GradeInfo(150, 30),
        "3a": GradeInfo(180, 30),
        "3b": GradeInfo(210, 30),
        "3c": GradeInfo(240, 30),
        "4a": GradeInfo(270, 30),
        "4b": GradeInfo(300, 99),
        "4c": GradeInfo(399, 102),
        "5a": GradeInfo(501, 99),
        "5b": GradeInfo(600, 99),
        "5c": GradeInfo(699, 102),
        "6a": GradeInfo(801, 99),
        "6b": GradeInfo(900, 99),
        "6c": GradeInfo(999, 102),
        "7a": GradeInfo(1101, 99),
        "7b": GradeInfo(1200, 102),
        "7c": GradeInfo(1302, 99),
    },
    "UIAA": {
        "1-": GradeInfo(0, 27),
        "1": GradeInfo(27, 27),
        "1+": GradeInfo(54, 27),
        "2-": GradeInfo(81, 27),
        "2": GradeInfo(108, 27),
        "2+": GradeInfo(135, 30),
        "3-": GradeInfo(165, 27),
        "3": GradeInfo(192, 27),
        "3+": GradeInfo(219, 27),
        "4-": GradeInfo(246, 27),
        "4": GradeInfo(273, 27),
        "4+": GradeInfo(300, 42),
        "5-": GradeInfo(342, 45),
        "5": GradeInfo(387, 42),
        "5+": GradeInfo(429, 42),
        "6-": GradeInfo(471, 42),
        "6": GradeInfo(513, 45),
        "6+": GradeInfo(558, 42),
        "7-": GradeInfo(600, 54),
        "7": GradeInfo(654, 54),
        "7+": GradeInfo(708, 54),
        "8-": GradeInfo(762, 54),
        "8": GradeInfo(816, 54),
        "8+": GradeInfo(870, 54),
        "9-": GradeInfo(924, 45),
        "9": GradeInfo(969, 48),
        "9+": GradeInfo(1017, 45),
        "10-": GradeInfo(1062, 45),
        "10": GradeInfo(1107, 48),
        "10+": GradeInfo(1155, 45),
        "11-": GradeInfo(1200, 51),
        "11": GradeInfo(1251, 48),
        "11+": GradeInfo(1299, 51),
        "12-": GradeInfo(1350, 51),
        "12": GradeInfo(1401, 48),
        "12+": GradeInfo(1449, 51),
    },
    "South African": {
        "1": GradeInfo(0, 24),
        "2": GradeInfo(24, 27),
        "3": GradeInfo(51, 24),
        "4": GradeInfo(75, 24),
        "5": GradeInfo(99, 27),
        "6": GradeInfo(126, 24),
        "7": GradeInfo(150, 24),
        "8": GradeInfo(174, 27),
        "9": GradeInfo(201, 24),
        "10": GradeInfo(225, 24),
        "11": GradeInfo(249, 27),
        "12": GradeInfo(276, 24),
        "13": GradeInfo(300, 42),
        "14": GradeInfo(342, 45),
        "15": GradeInfo(387, 42),
        "16": GradeInfo(429, 42),
        "17": GradeInfo(471, 42),
        "18": GradeInfo(513, 45),
        "19": GradeInfo(558, 42),
        "20": GradeInfo(600, 42),
        "21": GradeInfo(642, 45),
        "22": GradeInfo(687, 42),
        "23": GradeInfo(729, 42),
        "24": GradeInfo(771, 42),
        "25": GradeInfo(813, 45),
        "26": GradeInfo(858, 42),
        "27": GradeInfo(900, 39),
        "28": GradeInfo(939, 36),
        "29": GradeInfo(975, 39),
        "30": GradeInfo(1014, 36),
        "31": GradeInfo(1050, 39),
        "32": GradeInfo(1089, 36),
        "33": GradeInfo(1125, 39),
        "34": GradeInfo(1164, 36),
        "35": GradeInfo(1200, 42),
        "36": GradeInfo(1242, 45),
        "37": GradeInfo(1287, 42),
        "38": GradeInfo(1329, 42),
        "39": GradeInfo(1371, 42),
        "40": GradeInfo(1413, 45),
    },
    "Old South African": {
        "A1": GradeInfo(0, 18),
        "A2": GradeInfo(18, 21),
        "A3": GradeInfo(39, 18),
        "B1": GradeInfo(57, 18),
        "B2": GradeInfo(75, 18),
        "B3": GradeInfo(93, 21),
        "C1": GradeInfo(114, 18),
        "C2": GradeInfo(132, 18),
        "C3": GradeInfo(150, 18),
        "D1": GradeInfo(168, 21),
        "D2": GradeInfo(189, 18),
        "D3": GradeInfo(207, 18),
        "E1": GradeInfo(225, 18),
        "E2": GradeInfo(243, 21),
        "E3": GradeInfo(264, 18),
        "F1": GradeInfo(282, 18),
        "F2": GradeInfo(300, 75),
        "F3": GradeInfo(375, 75),
        "G1": GradeInfo(450, 75),
        "G2": GradeInfo(525, 75),
        "G3": GradeInfo(600, 75),
        "H1": GradeInfo(675, 75),
        "H2": GradeInfo(750, 75),
        "H3": GradeInfo(825, 75),
        "I1": GradeInfo(900, 75),
        "I2": GradeInfo(975, 75),
        "I3": GradeInfo(1050, 75),
        "J1": GradeInfo(1125, 75),
        "J2": GradeInfo(1200, 45),
        "J3": GradeInfo(1245, 42),
    },
    "Saxon": {
        "I": GradeInfo(0, 99),
        "II": GradeInfo(99, 102),
        "III": GradeInfo(201, 99),
        "IV": GradeInfo(300, 51),
        "V": GradeInfo(351, 48),
        "VI": GradeInfo(399, 51),
        "VIIa": GradeInfo(450, 51),
        "VIIb": GradeInfo(501, 48),
        "VIIc": GradeInfo(549, 51),
        "VIIIa": GradeInfo(600, 51),
        "VIIIb": GradeInfo(651, 48),
        "VIIIc": GradeInfo(699, 51),
        "IXa": GradeInfo(750, 51),
        "IXb": GradeInfo(801, 48),
        "IXc": GradeInfo(849, 51),
        "Xa": GradeInfo(900, 51),
        "Xb": GradeInfo(951, 48),
        "Xc": GradeInfo(999, 51),
        "XIa": GradeInfo(1050, 51),
        "XIb": GradeInfo(1101, 48),
        "XIc": GradeInfo(1149, 51),
        "XIIa": GradeInfo(1200, 51),
        "XIIb": GradeInfo(1251, 48),
        "XIIc": GradeInfo(1299, 51),
        "XIIIa": GradeInfo(1350, 51),
        "XIIIb": GradeInfo(1401, 48),
        "XIIIc": GradeInfo(1449, 51),
    },
    "Finnish": {
        "1-": GradeInfo(0, 27),
        "1": GradeInfo(27, 27),
        "1+": GradeInfo(54, 27),
        "2-": GradeInfo(81, 27),
        "2": GradeInfo(108, 27),
        "2+": GradeInfo(135, 30),
        "3-": GradeInfo(165, 27),
        "3": GradeInfo(192, 27),
        "3+": GradeInfo(219, 27),
        "4-": GradeInfo(246, 27),
        "4": GradeInfo(273, 27),
        "4+": GradeInfo(300, 48),
        "5-": GradeInfo(348, 51),
        "5": GradeInfo(399, 48),
        "5+": GradeInfo(447, 87),
        "6-": GradeInfo(534, 84),
        "6": GradeInfo(618, 60),
        "6+": GradeInfo(678, 63),
        "7-": GradeInfo(741, 60),
        "7": GradeInfo(801, 63),
        "7+": GradeInfo(864, 60),
        "8-": GradeInfo(924, 30),
        "8": GradeInfo(954, 30),
        "8+": GradeInfo(984, 33),
        "9-": GradeInfo(1017, 30),
        "9": GradeInfo(1047, 30),
        "9+": GradeInfo(1077, 30),
        "10-": GradeInfo(1107, 33),
        "10": GradeInfo(1140, 30),
        "10+": GradeInfo(1170, 30),
        "11-": GradeInfo(1200, 51),
        "11": GradeInfo(1251, 48),
        "11+": GradeInfo(1299, 51),
        "12-": GradeInfo(1350, 51),
        "12": GradeInfo(1401, 48),
        "12+": GradeInfo(1449, 51),
    },
    "Norwegian": {
        "1-": GradeInfo(0, 27),
        "1": GradeInfo(27, 27),
        "1+": GradeInfo(54, 27),
        "2-": GradeInfo(81, 27),
        "2": GradeInfo(108, 27),
        "2+": GradeInfo(135, 30),
        "3-": GradeInfo(165, 27),
        "3": GradeInfo(192, 27),
        "3+": GradeInfo(219, 27),
        "4-": GradeInfo(246, 27),
        "4": GradeInfo(273, 27),
        "4+": GradeInfo(300, 48),
        "5-": GradeInfo(348, 51),
        "5": GradeInfo(399, 48),
        "5+": GradeInfo(447, 87),
        "6-": GradeInfo(534, 84),
        "6": GradeInfo(618, 51),
        "6+": GradeInfo(669, 51),
        "7-": GradeInfo(720, 51),
        "7": GradeInfo(771, 51),
        "7+": GradeInfo(822, 51),
        "8-": GradeInfo(873, 51),
        "8": GradeInfo(924, 66),
        "8+": GradeInfo(990, 63),
        "9-": GradeInfo(1053, 66),
        "9": GradeInfo(1119, 63),
        "9+": GradeInfo(1182, 54),
        "10-": GradeInfo(1236, 51),
        "10": GradeInfo(1287, 54),
        "10+": GradeInfo(1341, 54),
        "11-": GradeInfo(1395, 51),
        "11": GradeInfo(1446, 54),
        "11+": GradeInfo(1500, 24),
        "12-": GradeInfo(1524, 21),
        "12": GradeInfo(1545, 24),
        "12+": GradeInfo(1569, 21),
    },
    "Polish": {
        "I-": GradeInfo(0, 27),
        "I": GradeInfo(27, 27),
        "I+": GradeInfo(54, 27),
        "II-": GradeInfo(81, 27),
        "II": GradeInfo(108, 27),
        "II+": GradeInfo(135, 30),
        "III-": GradeInfo(165, 27),
        "III": GradeInfo(192, 27),
        "III+": GradeInfo(219, 27),
        "IV-": GradeInfo(246, 27),
        "IV": GradeInfo(273, 27),
        "IV+": GradeInfo(300, 51),
        "V-": GradeInfo(351, 48),
        "V": GradeInfo(399, 51),
        "V+": GradeInfo(450, 51),
        "VI-": GradeInfo(501, 48),
        "VI": GradeInfo(549, 51),
        "VI+": GradeInfo(600, 42),
        "VI.1": GradeInfo(642, 45),
        "VI.1+": GradeInfo(687, 42),
        "VI.2": GradeInfo(729, 42),
        "VI.2+": GradeInfo(771, 42),
        "VI.3": GradeInfo(813, 45),
        "VI.3+": GradeInfo(858, 42),
        "VI.4": GradeInfo(900, 51),
        "VI.4+": GradeInfo(951, 48),
        "VI.5": GradeInfo(999, 51),
        "VI.5+": GradeInfo(1050, 51),
        "VI.6": GradeInfo(1101, 48),
        "VI.6+": GradeInfo(1149, 51),
        "VI.7": GradeInfo(1200, 51),
        "VI.7+": GradeInfo(1251, 48),
        "VI.8": GradeInfo(1299, 51),
        "VI.8+": GradeInfo(1350, 51),
        "VI.9": GradeInfo(1401, 48),
        "VI.9+": GradeInfo(1449, 51),
    },
    "Brazil Technical": {
        "I": GradeInfo(0, 60),
        "Isup": GradeInfo(60, 60),
        "II": GradeInfo(120, 60),
        "IIsup": GradeInfo(180, 60),
        "III": GradeInfo(240, 60),
        "IIIsup": GradeInfo(300, 75),
        "IV": GradeInfo(375, 75),
        "IVsup": GradeInfo(450, 75),
        "V": GradeInfo(525, 75),
        "Vsup": GradeInfo(600, 42),
        "VI": GradeInfo(642, 45),
        "VIsup": GradeInfo(687, 42),
        "VIIa": GradeInfo(729, 42),
        "VIIb": GradeInfo(771, 45),
        "VIIc": GradeInfo(816, 42),
        "VIIIa": GradeInfo(858, 42),
        "VIIIb": GradeInfo(900, 39),
        "VIIIc": GradeInfo(939, 36),
        "IXa": GradeInfo(975, 39),
        "IXb": GradeInfo(1014, 36),
        "IXc": GradeInfo(1050, 39),
        "Xa": GradeInfo(1089, 36),
        "Xb": GradeInfo(1125, 39),
        "Xc": GradeInfo(1164, 36),
        "XIa": GradeInfo(1200, 42),
        "XIb": GradeInfo(1242, 45),
        "XIc": GradeInfo(1287, 42),
        "XIIa": GradeInfo(1329, 42),
        "XIIb": GradeInfo(1371, 45),
        "XIIc": GradeInfo(1416, 42),
    },
    "Swedish": {
        "1-": GradeInfo(0, 30),
        "1": GradeInfo(30, 30),
        "1+": GradeInfo(60, 30),
        "2-": GradeInfo(90, 30),
        "2": GradeInfo(120, 30),
        "2+": GradeInfo(150, 30),
        "3-": GradeInfo(180, 30),
        "3": GradeInfo(210, 30),
        "3+": GradeInfo(240, 30),
        "4-": GradeInfo(270, 30),
        "4": GradeInfo(300, 54),
        "4+": GradeInfo(354, 54),
        "5-": GradeInfo(408, 51),
        "5": GradeInfo(459, 54),
        "5+": GradeInfo(513, 45),
        "6-": GradeInfo(558, 42),
        "6": GradeInfo(600, 54),
        "6+": GradeInfo(654, 54),
        "7-": GradeInfo(708, 54),
        "7": GradeInfo(762, 54),
        "7+": GradeInfo(816, 54),
        "8-": GradeInfo(870, 57),
        "8": GradeInfo(927, 57),
        "8+": GradeInfo(984, 57),
        "9-": GradeInfo(1041, 57),
        "9": GradeInfo(1098, 57),
        "9+": GradeInfo(1155, 57),
        "10-": GradeInfo(1212, 57),
        "10": GradeInfo(1269, 60),
        "10+": GradeInfo(1329, 57),
        "11-": GradeInfo(1386, 57),
        "11": GradeInfo(1443, 57),
        "11+": GradeInfo(1500, 24),
        "12-": GradeInfo(1524, 21),
        "12": GradeInfo(1545, 24),
        "12+": GradeInfo(1569, 21),
    },
    "Russian": {
        "1A": GradeInfo(0, 51),
        "1B": GradeInfo(51, 48),
        "2A": GradeInfo(99, 51),
        "2B": GradeInfo(150, 51),
        "3A": GradeInfo(201, 48),
        "3B": GradeInfo(249, 51),
        "4A": GradeInfo(300, 150),
        "4B": GradeInfo(450, 150),
        "5A": GradeInfo(600, 150),
        "5B": GradeInfo(750, 150),
        "6A": GradeInfo(900, 150),
        "6B": GradeInfo(1050, 150),
        "7A": GradeInfo(1200, 150),
        "7B": GradeInfo(1350, 150),
    },
}


BOULDER_MAPPING = {
    "Band Boulder": {
        "Beginner": GradeInfo(0, 200),
        "Intermediate": GradeInfo(200, 200),
        "Experienced": GradeInfo(400, 200),
        "Expert": GradeInfo(600, 200),
        "Elite": GradeInfo(800, 300),
    },
    "V-Scale": {
        "VB-": GradeInfo(0, 66),
        "VB": GradeInfo(66, 68),
        "VB+": GradeInfo(134, 66),
        "V0-": GradeInfo(200, 66),
        "V0": GradeInfo(266, 68),
        "V0+": GradeInfo(334, 66),
        "V1": GradeInfo(400, 50),
        "V2": GradeInfo(450, 50),
        "V3": GradeInfo(500, 50),
        "V4": GradeInfo(550, 50),
        "V5": GradeInfo(600, 34),
        "V6": GradeInfo(634, 32),
        "V7": GradeInfo(666, 34),
        "V8": GradeInfo(700, 34),
        "V9": GradeInfo(734, 32),
        "V10": GradeInfo(766, 34),
        "V11": GradeInfo(800, 34),
        "V12": GradeInfo(834, 32),
        "V13": GradeInfo(866, 34),
        "V14": GradeInfo(900, 34),
        "V15": GradeInfo(934, 34),
        "V16": GradeInfo(968, 32),
        "V17": GradeInfo(1000, 34),
    },
    "B-Scale": {
        "B5.6": GradeInfo(200, 40),
        "B5.7": GradeInfo(240, 40),
        "B5.8": GradeInfo(280, 40),
        "B5.9": GradeInfo(320, 40),
        "B5.10": GradeInfo(360, 88),
        "B5.10+": GradeInfo(448, 28),
        "B1-": GradeInfo(476, 42),
        "B1": GradeInfo(518, 40),
        "B1+": GradeInfo(558, 42),
        "B2-": GradeInfo(600, 66),
        "B2": GradeInfo(666, 68),
        "B2+": GradeInfo(734, 66),
        "B3": GradeInfo(800, 100),
    },
    "S-Scale": {
        "S1-": GradeInfo(400, 26),
        "S1": GradeInfo(426, 24),
        "S1+": GradeInfo(450, 26),
        "S2-": GradeInfo(476, 24),
        "S2": GradeInfo(500, 26),
        "S2+": GradeInfo(526, 24),
        "S3-": GradeInfo(550, 26),
        "S3": GradeInfo(576, 24),
        "S3+": GradeInfo(600, 26),
        "S4-": GradeInfo(626, 24),
        "S4": GradeInfo(650, 26),
        "S4+": GradeInfo(676, 24),
        "S5-": GradeInfo(700, 26),
        "S5": GradeInfo(726, 24),
        "S5+": GradeInfo(750, 26),
        "S6-": GradeInfo(776, 24),
        "S6": GradeInfo(800, 34),
        "S6+": GradeInfo(834, 32),
    },
    "P-Scale": {
        "P0": GradeInfo(200, 50),
        "P1": GradeInfo(250, 50),
        "P2": GradeInfo(300, 50),
        "P3": GradeInfo(350, 50),
        "P4": GradeInfo(400, 50),
        "P5": GradeInfo(450, 50),
        "P6": GradeInfo(500, 50),
        "P7": GradeInfo(550, 50),
        "P8": GradeInfo(600, 50),
        "P9": GradeInfo(650, 50),
        "P10": GradeInfo(700, 50),
        "P11": GradeInfo(750, 50),
        "P12": GradeInfo(800, 34),
        "P13": GradeInfo(834, 34),
        "P14": GradeInfo(868, 32),
        "P15": GradeInfo(900, 34),
        "P16": GradeInfo(934, 34),
    },
    "Joshua Tree Scale": {
        "A-": GradeInfo(400, 40),
        "A": GradeInfo(440, 40),
        "A+": GradeInfo(480, 40),
        "B-": GradeInfo(520, 40),
        "B": GradeInfo(560, 40),
        "B+": GradeInfo(600, 40),
        "C-": GradeInfo(640, 40),
        "C": GradeInfo(680, 40),
        "C+": GradeInfo(720, 40),
        "D-": GradeInfo(760, 40),
        "D": GradeInfo(800, 34),
        "D+": GradeInfo(834, 32),
    },
    "Fontainebleau": {
        "1A": GradeInfo(0, 12),
        "1A+": GradeInfo(12, 10),
        "1B": GradeInfo(22, 12),
        "1B+": GradeInfo(34, 10),
        "1C": GradeInfo(44, 12),
        "1C+": GradeInfo(56, 10),
        "2A": GradeInfo(66, 12),
        "2A+": GradeInfo(78, 10),
        "2B": GradeInfo(88, 12),
        "2B+": GradeInfo(100, 12),
        "2C": GradeInfo(112, 10),
        "2C+": GradeInfo(122, 12),
        "3A": GradeInfo(134, 10),
        "3A+": GradeInfo(144, 12),
        "3B": GradeInfo(156, 10),
        "3B+": GradeInfo(166, 12),
        "3C": GradeInfo(178, 10),
        "3C+": GradeInfo(188, 12),
        "4A": GradeInfo(200, 34),
        "4A+": GradeInfo(234, 32),
        "4B": GradeInfo(266, 34),
        "4B+": GradeInfo(300, 34),
        "4C": GradeInfo(334, 32),
        "4C+": GradeInfo(366, 34),
        "5A": GradeInfo(400, 20),
        "5A+": GradeInfo(420, 20),
        "5B": GradeInfo(440, 20),
        "5B+": GradeInfo(460, 20),
        "5C": GradeInfo(480, 20),
        "5C+": GradeInfo(500, 20),
        "6A": GradeInfo(520, 20),
        "6A+": GradeInfo(540, 20),
        "6B": GradeInfo(560, 20),
        "6B+": GradeInfo(580, 20),
        "6C": GradeInfo(600, 26),
        "6C+": GradeInfo(626, 24),
        "7A": GradeInfo(650, 26),
        "7A+": GradeInfo(676, 24),
        "7B": GradeInfo(700, 26),
        "7B+": GradeInfo(726, 24),
        "7C": GradeInfo(750, 26),
        "7C+": GradeInfo(776, 24),
        "8A": GradeInfo(800, 34),
        "8A+": GradeInfo(834, 32),
        "8B": GradeInfo(866, 34),
        "8B+": GradeInfo(900, 34),
        "8C": GradeInfo(934, 34),
        "8C+": GradeInfo(968, 32),
        "9A": GradeInfo(1000, 34),
        "9A+": GradeInfo(1034, 14),
        "9B": GradeInfo(1048, 12),
        "9B+": GradeInfo(1060, 14),
        "9C": GradeInfo(1074, 12),
        "9C+": GradeInfo(1086, 14),
    },
    "Annot B-Scale": {
        "B1": GradeInfo(200, 100),
        "B2": GradeInfo(300, 100),
        "B3": GradeInfo(400, 50),
        "B4": GradeInfo(450, 50),
        "B5": GradeInfo(500, 50),
        "B6": GradeInfo(550, 50),
        "B7": GradeInfo(600, 28),
        "B8": GradeInfo(628, 30),
        "B9": GradeInfo(658, 28),
        "B10": GradeInfo(686, 28),
        "B11": GradeInfo(714, 28),
        "B12": GradeInfo(742, 30),
        "B13": GradeInfo(772, 28),
        "B14": GradeInfo(800, 56),
        "B15": GradeInfo(856, 56),
        "B16": GradeInfo(912, 56),
    },
    "Font Traverse": {
        "1A": GradeInfo(0, 12),
        "1A+": GradeInfo(12, 10),
        "1B": GradeInfo(22, 12),
        "1B+": GradeInfo(34, 10),
        "1C": GradeInfo(44, 12),
        "1C+": GradeInfo(56, 10),
        "2A": GradeInfo(66, 12),
        "2A+": GradeInfo(78, 10),
        "2B": GradeInfo(88, 12),
        "2B+": GradeInfo(100, 12),
        "2C": GradeInfo(112, 10),
        "2C+": GradeInfo(122, 12),
        "3A": GradeInfo(134, 10),
        "3A+": GradeInfo(144, 12),
        "3B": GradeInfo(156, 10),
        "3B+": GradeInfo(166, 12),
        "3C": GradeInfo(178, 10),
        "3C+": GradeInfo(188, 12),
        "4A": GradeInfo(200, 16),
        "4A+": GradeInfo(216, 14),
        "4B": GradeInfo(230, 16),
        "4B+": GradeInfo(246, 16),
        "4C": GradeInfo(262, 14),
        "4C+": GradeInfo(276, 16),
        "5A": GradeInfo(292, 16),
        "5A+": GradeInfo(308, 16),
        "5B": GradeInfo(324, 14),
        "5B+": GradeInfo(338, 16),
        "5C": GradeInfo(354, 16),
        "5C+": GradeInfo(370, 14),
        "6A": GradeInfo(384, 16),
        "6A+": GradeInfo(400, 28),
        "6B": GradeInfo(428, 30),
        "6B+": GradeInfo(458, 28),
        "6C": GradeInfo(486, 28),
        "6C+": GradeInfo(514, 28),
        "7A": GradeInfo(542, 30),
        "7A+": GradeInfo(572, 28),
        "7B": GradeInfo(600, 26),
        "7B+": GradeInfo(626, 24),
        "7C": GradeInfo(650, 26),
        "7C+": GradeInfo(676, 24),
        "8A": GradeInfo(700, 26),
        "8A+": GradeInfo(726, 24),
        "8B": GradeInfo(750, 26),
        "8B+": GradeInfo(776, 24),
        "8C": GradeInfo(800, 28),
        "8C+": GradeInfo(828, 30),
        "9A": GradeInfo(858, 28),
        "9A+": GradeInfo(886, 28),
        "9B": GradeInfo(914, 28),
        "9B+": GradeInfo(942, 30),
        "9C": GradeInfo(972, 28),
    },
}

OTHER_MAPPING = {
    "Band Other": {
        "Beginner": GradeInfo(0, 300),
        "Intermediate": GradeInfo(300, 300),
        "Experienced": GradeInfo(600, 300),
        "Expert": GradeInfo(900, 300),
        "Elite": GradeInfo(1200, 450),
    },
    "Aid": {
        "C0": GradeInfo(0, 99),
        "C0+": GradeInfo(99, 102),
        "C1": GradeInfo(201, 99),
        "C1+": GradeInfo(300, 99),
        "C2": GradeInfo(399, 102),
        "C2+": GradeInfo(501, 99),
        "C3": GradeInfo(600, 150),
        "C3+": GradeInfo(750, 150),
        "C4": GradeInfo(900, 150),
        "C4+": GradeInfo(1050, 150),
        "C5": GradeInfo(1200, 99),
        "C5+": GradeInfo(1299, 102),
        "C6": GradeInfo(1401, 99),
    },
    "Alpine Ice": {
        "AI1-": GradeInfo(0, 42),
        "AI1": GradeInfo(42, 45),
        "AI1+": GradeInfo(87, 42),
        "AI2-": GradeInfo(129, 42),
        "AI2": GradeInfo(171, 42),
        "AI2+": GradeInfo(213, 45),
        "AI3-": GradeInfo(258, 42),
        "AI3": GradeInfo(300, 60),
        "AI3+": GradeInfo(360, 60),
        "AI4-": GradeInfo(420, 60),
        "AI4": GradeInfo(480, 60),
        "AI4+": GradeInfo(540, 60),
        "AI5-": GradeInfo(600, 75),
        "AI5": GradeInfo(675, 75),
        "AI5+": GradeInfo(750, 75),
        "AI6-": GradeInfo(825, 75),
        "AI6": GradeInfo(900, 75),
        "AI6+": GradeInfo(975, 75),
        "AI7-": GradeInfo(1050, 75),
        "AI7": GradeInfo(1125, 75),
        "AI7+": GradeInfo(1200, 21),
        "AI8-": GradeInfo(1221, 21),
        "AI8": GradeInfo(1242, 18),
        "AI8+": GradeInfo(1260, 21),
    },
    "Water Ice": {
        "WI1-": GradeInfo(0, 42),
        "WI1": GradeInfo(42, 45),
        "WI1+": GradeInfo(87, 42),
        "WI2-": GradeInfo(129, 42),
        "WI2": GradeInfo(171, 42),
        "WI2+": GradeInfo(213, 45),
        "WI3-": GradeInfo(258, 42),
        "WI3": GradeInfo(300, 60),
        "WI3+": GradeInfo(360, 60),
        "WI4-": GradeInfo(420, 60),
        "WI4": GradeInfo(480, 60),
        "WI4+": GradeInfo(540, 60),
        "WI5-": GradeInfo(600, 75),
        "WI5": GradeInfo(675, 75),
        "WI5+": GradeInfo(750, 75),
        "WI6-": GradeInfo(825, 75),
        "WI6": GradeInfo(900, 75),
        "WI6+": GradeInfo(975, 75),
        "WI7-": GradeInfo(1050, 75),
        "WI7": GradeInfo(1125, 75),
        "WI7+": GradeInfo(1200, 21),
        "WI8-": GradeInfo(1221, 21),
        "WI8": GradeInfo(1242, 18),
        "WI8+": GradeInfo(1260, 21),
        "WI9-": GradeInfo(1281, 21),
        "WI9": GradeInfo(1302, 21),
        "WI9+": GradeInfo(1323, 21),
        "WI10-": GradeInfo(1344, 21),
        "WI10": GradeInfo(1365, 18),
        "WI10+": GradeInfo(1383, 21),
        "WI11-": GradeInfo(1404, 21),
        "WI11": GradeInfo(1425, 21),
        "WI11+": GradeInfo(1446, 21),
        "WI12-": GradeInfo(1467, 18),
        "WI12": GradeInfo(1485, 21),
        "WI12+": GradeInfo(1506, 21),
        "WI13-": GradeInfo(1527, 21),
        "WI13": GradeInfo(1548, 21),
        "WI13+": GradeInfo(1569, 21),
        "WI14-": GradeInfo(1590, 18),
        "WI14": GradeInfo(1608, 21),
        "WI14+": GradeInfo(1629, 21),
    },
    "Mixed Rock/Ice": {
        "M1": GradeInfo(66, 117),
        "M2": GradeInfo(183, 117),
        "M3": GradeInfo(300, 99),
        "M4": GradeInfo(399, 102),
        "M5": GradeInfo(501, 99),
        "M6": GradeInfo(600, 150),
        "M7": GradeInfo(750, 150),
        "M8": GradeInfo(900, 150),
        "M9": GradeInfo(1050, 150),
        "M10": GradeInfo(1200, 99),
        "M11": GradeInfo(1299, 102),
        "M12": GradeInfo(1401, 99),
    },
    "Ferrata Schall": {
        "A": GradeInfo(0, 99),
        "B": GradeInfo(99, 102),
        "C": GradeInfo(201, 99),
        "D": GradeInfo(300, 300),
        "E": GradeInfo(600, 300),
        "F": GradeInfo(900, 300),
    },
    "Ferrata Num": {
        "1": GradeInfo(0, 99),
        "2": GradeInfo(99, 102),
        "3": GradeInfo(201, 99),
        "4": GradeInfo(300, 300),
        "5": GradeInfo(600, 300),
        "6": GradeInfo(900, 300),
    },
    "Ferrata French": {
        "F": GradeInfo(0, 99),
        "PD": GradeInfo(99, 102),
        "AD": GradeInfo(201, 99),
        "D": GradeInfo(300, 300),
        "TD": GradeInfo(600, 300),
        "ED": GradeInfo(900, 300),
    },
    "Scottish Winter Technical": {
        "1": GradeInfo(0, 75),
        "2": GradeInfo(75, 75),
        "3": GradeInfo(150, 75),
        "4": GradeInfo(225, 75),
        "5": GradeInfo(300, 300),
        "6": GradeInfo(600, 150),
        "7": GradeInfo(750, 150),
        "8": GradeInfo(900, 300),
        "9": GradeInfo(1200, 99),
        "10": GradeInfo(1299, 102),
        "11": GradeInfo(1401, 99),
    },
}
